obj/levelAsset
	//a levelAsset is pretty much anything that appears on a level. Most of them have pretty consistent behaviors
	//from level to level (well, not really, but kind of), so we can save a lot of lines by identifying common
	//behaviors and handling them in parent types.
	var/initialInvisibility = 0 //use 100 for fully invisible
	var/visibleOnActivate = 1


	proc/Reset()
		invisibility = initialInvisibility


	proc/Activate(obj/levelAsset/activator)
		//a key, when touched, can automatically call Activate on all objects of a single type, via its "activates" var.
		//of course a key can be made to do more, but that's not so automatic.
		if(visibleOnActivate)
			invisibility = 0


	entrance
		icon = 'entrance.dmi'
		var/seconds = DEFAULT_SECONDS


		Reset()
			. = ..()
			seconds = initial(seconds)


		MouseEntered()
			//if the level isn't already started, start it when pointer touches cage.
			if(!gameOn)
				gameOn = 1
				invisibility = 100
				spawn HandleTimer()


		proc/HandleTimer()
			while(1)
				if(!gameOn) break

				thePlayer.timer.Update(round(seconds + 0.9))

				if(seconds <= 0)
					gameOn = 0
					MockFailure()
					ResetLevel(src.z)
					return

				sleep(TIMER_SLEEP)
				seconds -= TIMER_SLEEP / 10
				seconds = round(seconds, 0.1)


	obstacle
		icon = 'obstacle.dmi'
		var/deadly = 1 //not all obstacles are deadly. Some are just for show.


		MouseEntered()
			if(!gameOn) return

			if(deadly)
				gameOn = 0
				MockFailure()
				ResetLevel(src.z)


		wall
			icon = 'wall.dmi'


	key
		layer = OBJ_LAYER - 0.1
		icon = 'key.dmi'
		var/activates //type path of level assets to be activated by picking up key
		var/vanishOnEntry = 1
		var/usesPerRound = 1 //prevent a key from re-activating things over and over again
		var/uses


		MouseEntered()
			if(!gameOn) return

			if(++uses > usesPerRound) return

			if(vanishOnEntry)
				invisibility = 100 //make key disappear so it doesn't get activated again

			for(var/obj/levelAsset/LA in levelAssets[src.z])
				if(istype(LA, activates))
					LA.Activate(src)


		Reset()
			. = ..()
			uses = 0


	exit
		layer = OBJ_LAYER + 0.1
		icon = 'exit.dmi'
		mouse_opacity = 2


		MouseEntered()
			if(!gameOn) return

			Congratulate()

			thePlayer.SetLevel(thePlayer.z + 1)


//BELOW: CUSTOM DEFINITIONS OF LEVEL-SPECIFIC ASSETS
obj/levelAsset/key
	litter //level 1
		icon_state = "litter"
		activates = /obj/levelAsset/exit/trash


	stripe //level 2
		icon_state = "stripe_horiz"
		activates = /obj/levelAsset/key/stripe
		initialInvisibility = 100
		vanishOnEntry = 0
		mouse_opacity = 2 //whole icon is sensitive, not just stripes


		Activate(myActivator)
			//when touched, a stripe tries to activate all the stripes in the level.
			//each stripe must determine if it is close enough to become visible.
			if(get_dist(myActivator, src) <= 1)
				invisibility = 0


		first_stripe //this is the one adjacent to the entrance. It is always visible.
			initialInvisibility = 0


	spring_roll //level 3
		icon_state = "spring roll"
		activates = /obj/levelAsset/exit/cookie


	pressure_pad_4 //level 4
		icon_state = "key"
		mouse_opacity = 2
		activates = /obj/levelAsset/obstacle/sign


		Reset()
			. = ..()
			//we want them visible to us in the map editor; this code makes them invisible to the player at runtime.
			//can't use initialInvisibility for this, as it will make the pad untouchable.
			icon_state = "invisible"


	pressure_pad_6 //level 6
		icon_state = "key"
		mouse_opacity = 2
		activates = /obj/levelAsset/obstacle/smoke


		Reset()
			. = ..()
			icon_state = "invisible"


	chair //level 5
		vanishOnEntry = 0
		icon_state = "chair"
		activates = /obj/levelAsset/obstacle/spikes


obj/levelAsset/exit
	trash //level 1
		initialInvisibility = 100
		icon_state = "trash"


	cookie //level 3
		initialInvisibility = 100
		icon_state = "cookie"


obj/levelAsset/entrance
	level3
		MouseEntered()
			. = ..()
			//make a list of all the takeout boxes.
			var/list/boxes = list()
			var/obj/levelAsset/obstacle/takeout_box/T
			for(T in levelAssets[src.z])
				boxes += T

			//pick a box and hide the spring roll in it.
			var/obj/levelAsset/key/spring_roll/S = locate() in levelAssets[src.z]
			T = pick(boxes)
			S.loc = T.loc


obj/levelAsset/obstacle
	invisible //level 2; barriers that kill player for straying from the "road".
		mouse_opacity = 2 //fill the whole turf

		New()
			. = ..()
			icon_state = "invisible"


	takeout_box //level 3
		deadly = 0
		icon_state = "takeout"
		mouse_opacity = 0


	grease //level 3
		icon_state = "grease1"


		Reset()
			. = ..()
			//at runtime, randomly give each grease object one of three icon states
			icon_state = pick(list("grease1", "grease2", "grease3"))


	sign //level 4
		icon_state = "sign1"
		var/jibbly = 0
		var/xmod = SIGN_XMOD


		Activate()
			if(!jibbly)
				jibbly = 1
				spawn Jibble()


		Reset()
			. = ..()
			icon_state = pick(list("sign1", "sign2", "sign3", "sign4"))
			pixel_x = rand(-8, 8)
			jibbly = 0
			xmod = SIGN_XMOD


		proc/Jibble()
			//move the sign back and forth
			while(jibbly)
				pixel_x += xmod
				if(pixel_x >= MAX_SIGN_PIXELX)
					pixel_x = MAX_SIGN_PIXELX
					xmod = -xmod
				else if(pixel_x <= -MAX_SIGN_PIXELX)
					pixel_x = -MAX_SIGN_PIXELX
					xmod = -xmod

				sleep(SIGN_DELAY)


	spikes //level 5
		icon_state = "spikeson"


		Reset()
			. = ..()
			deadly = 1
			icon_state = "spikeson"
			mouse_opacity = 1


		Activate()
			var/session = currentRound

			spawn(SPIKE_SLEEP)
				if(session == currentRound)
					icon_state = "spikesoff"
					deadly = 0
					mouse_opacity = 0 //this prevents irrelevant events (passing over holes when escaping room) from queueing up


	smoke //level 6
		var/jibbly = 0
		icon_state = "smoke"


		Activate()
			if(!jibbly)
				jibbly = 1
				spawn Jibble()


		Reset()
			. = ..()
			jibbly = 0
			loc = initial(loc)
			dir = initial(dir)


		proc/Jibble()
			while(jibbly)
				var/turf/T = get_step(src, src.dir)
				Move(T)
				if(locate(/obj/levelAsset/obstacle/turn) in T)
					dir = turn(dir, -90)

				sleep(SMOKE_DELAY)


		smoke_n
			//I could have edited the smoke dirs in the map editor instead of creating a second smoke type, but this was quicker.
			name = "smoke"
			dir = NORTH


	turn //level 6
		//this just tells smoke where to turn.
		deadly = 0
		initialInvisibility = 100
		icon_state = "turn"
